@@ -1,9 +1,10 @@
 /*
- * Copyright 1999-2004 The Apache Software Foundation.
- * 
- * Licensed under the Apache License, Version 2.0 (the "License");
- * you may not use this file except in compliance with the License.
- * You may obtain a copy of the License at
+ * Licensed to the Apache Software Foundation (ASF) under one or more
+ * contributor license agreements.  See the NOTICE file distributed with
+ * this work for additional information regarding copyright ownership.
+ * The ASF licenses this file to You under the Apache License, Version 2.0
+ * (the "License"); you may not use this file except in compliance with
+ * the License.  You may obtain a copy of the License at
  * 
  *      http://www.apache.org/licenses/LICENSE-2.0
  * 
@@ -16,25 +17,26 @@
 
 package org.apache.commons.pool.impl;
 
-import java.util.NoSuchElementException;
-import java.util.List;
-import java.util.ArrayList;
-import java.util.Arrays;
-
 import junit.framework.Test;
 import junit.framework.TestSuite;
-
 import org.apache.commons.pool.BasePoolableObjectFactory;
 import org.apache.commons.pool.ObjectPool;
 import org.apache.commons.pool.PoolableObjectFactory;
-import org.apache.commons.pool.TestObjectPool;
+import org.apache.commons.pool.PoolUtils;
+import org.apache.commons.pool.TestBaseObjectPool;
+import org.apache.commons.pool.VisitTracker;
+import org.apache.commons.pool.VisitTrackerFactory;
+
+import java.util.NoSuchElementException;
+import java.util.Random;
 
 /**
  * @author Rodney Waldhoff
  * @author Dirk Verbeeck
- * @version $Revision: 387103 $ $Date: 2006-03-19 20:48:49 -0500 (Sun, 19 Mar 2006) $
+ * @author Sandy McArthur
+ * @version $Revision: 609415 $ $Date: 2008-01-06 14:45:32 -0700 (Sun, 06 Jan 2008) $
  */
-public class TestGenericObjectPool extends TestObjectPool {
+public class TestGenericObjectPool extends TestBaseObjectPool {
     public TestGenericObjectPool(String testName) {
         super(testName);
     }
@@ -49,7 +51,11 @@ public class TestGenericObjectPool extends TestObjectPool {
        pool.setMaxIdle(mincap);
        return pool;
     }
-    
+
+    protected ObjectPool makeEmptyPool(final PoolableObjectFactory factory) {
+        return new GenericObjectPool(factory);
+    }
+
     protected Object getNthObject(int n) {
         return String.valueOf(n);
     }
@@ -65,45 +71,7 @@ public class TestGenericObjectPool extends TestObjectPool {
         pool = null;
     }
 
-    /**
-     * Activation failure on existing object doesn't fail the borrow 
-     */
-    public void testActivationException() throws Exception {
-        SimpleFactory factory = new SimpleFactory(true, false);
-        factory.setThrowExceptionOnActivate(true);
-        factory.setValidationEnabled(false);
-        GenericObjectPool pool = new GenericObjectPool(factory);
-        
-        Object obj1 = pool.borrowObject();
-        pool.returnObject(obj1);
-
-        // obj1 was returned to the pool but failed to activate the second borrow
-        // a new object obj2 needs te be created
-        Object obj2 = pool.borrowObject();
-        assertTrue(obj1 != obj2);        
-    }
-
-    /**
-     * Activation failure on new object fails the borrow 
-     */
-    public void testActivationExceptionOnNewObject() throws Exception {
-        SimpleFactory factory = new SimpleFactory(true, false);
-        factory.setThrowExceptionOnActivate(true);
-        factory.setValidationEnabled(false);
-        GenericObjectPool pool = new GenericObjectPool(factory);
-        
-        Object obj1 = pool.borrowObject();
-        try {
-            Object obj2 = pool.borrowObject();
-            System.out.println("obj1: " + obj1);
-            System.out.println("obj2: " + obj2);
-            fail("a second object should have been created and failed to activate");
-        }
-        catch (Exception e) {}
-    }
-
     public void testWhenExhaustedGrow() throws Exception {
-        GenericObjectPool pool = new GenericObjectPool(new SimpleFactory());
         pool.setMaxActive(1);
         pool.setWhenExhaustedAction(GenericObjectPool.WHEN_EXHAUSTED_GROW);
         Object obj1 = pool.borrowObject();
@@ -116,7 +84,6 @@ public class TestGenericObjectPool extends TestObjectPool {
     }
 
     public void testWhenExhaustedFail() throws Exception {
-        GenericObjectPool pool = new GenericObjectPool(new SimpleFactory());
         pool.setMaxActive(1);
         pool.setWhenExhaustedAction(GenericObjectPool.WHEN_EXHAUSTED_FAIL);
         Object obj1 = pool.borrowObject();
@@ -132,7 +99,6 @@ public class TestGenericObjectPool extends TestObjectPool {
     }
 
     public void testWhenExhaustedBlock() throws Exception {
-        GenericObjectPool pool = new GenericObjectPool(new SimpleFactory());
         pool.setMaxActive(1);
         pool.setWhenExhaustedAction(GenericObjectPool.WHEN_EXHAUSTED_BLOCK);
         pool.setMaxWait(10L);
@@ -149,11 +115,231 @@ public class TestGenericObjectPool extends TestObjectPool {
     }
 
     public void testEvictWhileEmpty() throws Exception {
-        GenericObjectPool pool = new GenericObjectPool(new SimpleFactory(true,false));
         pool.evict();
         pool.evict();
         pool.close();
     }
+    
+    /**
+     * Tests addObject contention between ensureMinIdle triggered by
+     * the Evictor with minIdle > 0 and borrowObject. 
+     */
+    public void testEvictAddObjects() throws Exception {
+        SimpleFactory factory = new SimpleFactory();
+        factory.setMakeLatency(300);
+        factory.setMaxActive(2);
+        GenericObjectPool pool = new GenericObjectPool(factory);
+        pool.setMaxActive(2);
+        pool.setMinIdle(1);
+        Object obj = pool.borrowObject(); // numActive = 1, numIdle = 0
+        // Create a test thread that will run once and try a borrow after
+        // 150ms fixed delay
+        TestThread borrower = new TestThread(pool, 1, 150, false);
+        Thread borrowerThread = new Thread(borrower);
+        // Set evictor to run in 100 ms - will create idle instance
+        pool.setTimeBetweenEvictionRunsMillis(100);
+        borrowerThread.start();  // Off to the races
+        borrowerThread.join();
+        assertTrue(!borrower.failed());
+        pool.close();
+    }
+
+    public void testEvictLIFO() throws Exception {
+        checkEvict(true);   
+    }
+    
+    public void testEvictFIFO() throws Exception {
+        checkEvict(false);
+    }
+    
+    public void checkEvict(boolean lifo) throws Exception {
+        // yea this is hairy but it tests all the code paths in GOP.evict()
+        final SimpleFactory factory = new SimpleFactory();
+        final GenericObjectPool pool = new GenericObjectPool(factory);
+        pool.setSoftMinEvictableIdleTimeMillis(10);
+        pool.setMinIdle(2);
+        pool.setTestWhileIdle(true);
+        pool.setLifo(lifo);
+        PoolUtils.prefill(pool, 5);
+        pool.evict();
+        factory.setEvenValid(false);
+        factory.setOddValid(false);
+        factory.setThrowExceptionOnActivate(true);
+        pool.evict();
+        PoolUtils.prefill(pool, 5);
+        factory.setThrowExceptionOnActivate(false);
+        factory.setThrowExceptionOnPassivate(true);
+        pool.evict();
+        factory.setThrowExceptionOnPassivate(false);
+        factory.setEvenValid(true);
+        factory.setOddValid(true);
+        Thread.sleep(125);
+        pool.evict();
+        assertEquals(2, pool.getNumIdle());
+    }
+    
+    /**
+     * Test to make sure evictor visits least recently used objects first,
+     * regardless of FIFO/LIFO 
+     * 
+     * JIRA: POOL-86
+     */ 
+    public void testEvictionOrder() throws Exception {
+        checkEvictionOrder(false);
+        checkEvictionOrder(true);
+    }
+    
+    private void checkEvictionOrder(boolean lifo) throws Exception {
+        SimpleFactory factory = new SimpleFactory();
+        GenericObjectPool pool = new GenericObjectPool(factory);
+        pool.setNumTestsPerEvictionRun(2);
+        pool.setMinEvictableIdleTimeMillis(100);
+        pool.setLifo(lifo);
+        for (int i = 0; i < 5; i++) {
+            pool.addObject();
+            Thread.sleep(100);
+        }
+        // Order, oldest to youngest, is "0", "1", ...,"4"
+        pool.evict(); // Should evict "0" and "1"
+        Object obj = pool.borrowObject();
+        assertTrue("oldest not evicted", !obj.equals("0"));
+        assertTrue("second oldest not evicted", !obj.equals("1"));
+        // 2 should be next out for FIFO, 4 for LIFO
+        assertEquals("Wrong instance returned", lifo ? "4" : "2" , obj); 
+        
+        // Two eviction runs in sequence
+        factory = new SimpleFactory();
+        pool = new GenericObjectPool(factory);
+        pool.setNumTestsPerEvictionRun(2);
+        pool.setMinEvictableIdleTimeMillis(100);
+        pool.setLifo(lifo);
+        for (int i = 0; i < 5; i++) {
+            pool.addObject();
+            Thread.sleep(100);
+        }
+        pool.evict(); // Should evict "0" and "1"
+        pool.evict(); // Should evict "2" and "3"
+        obj = pool.borrowObject();
+        assertEquals("Wrong instance remaining in pool", "4", obj);     
+    }
+    
+    /**
+     * Verifies that the evictor visits objects in expected order
+     * and frequency. 
+     */
+    public void testEvictorVisiting() throws Exception {
+        checkEvictorVisiting(true);
+        checkEvictorVisiting(false);  
+    }
+    
+    private void checkEvictorVisiting(boolean lifo) throws Exception {
+        VisitTrackerFactory factory = new VisitTrackerFactory();
+        GenericObjectPool pool = new GenericObjectPool(factory);
+        pool.setNumTestsPerEvictionRun(2);
+        pool.setMinEvictableIdleTimeMillis(-1);
+        pool.setTestWhileIdle(true);
+        pool.setLifo(lifo);
+        pool.setTestOnReturn(false);
+        pool.setTestOnBorrow(false);
+        for (int i = 0; i < 8; i++) {
+            pool.addObject();
+        }
+        pool.evict(); // Visit oldest 2 - 0 and 1
+        Object obj = pool.borrowObject();
+        pool.returnObject(obj);
+        obj = pool.borrowObject();
+        pool.returnObject(obj);
+        //  borrow, return, borrow, return 
+        //  FIFO will move 0 and 1 to end
+        //  LIFO, 7 out, then in, then out, then in
+        pool.evict();  // Should visit 2 and 3 in either case
+        for (int i = 0; i < 8; i++) {
+            VisitTracker tracker = (VisitTracker) pool.borrowObject();    
+            if (tracker.getId() >= 4) {
+                assertEquals("Unexpected instance visited " + tracker.getId(),
+                        0, tracker.getValidateCount());
+            } else {
+                assertEquals("Instance " +  tracker.getId() + 
+                        " visited wrong number of times.",
+                        1, tracker.getValidateCount());
+            }
+        } 
+
+        factory = new VisitTrackerFactory();
+        pool = new GenericObjectPool(factory);
+        pool.setNumTestsPerEvictionRun(3);
+        pool.setMinEvictableIdleTimeMillis(-1);
+        pool.setTestWhileIdle(true);
+        pool.setLifo(lifo);
+        pool.setTestOnReturn(false);
+        pool.setTestOnBorrow(false);
+        for (int i = 0; i < 8; i++) {
+            pool.addObject();
+        }
+        pool.evict(); // 0, 1, 2
+        pool.evict(); // 3, 4, 5
+        obj = pool.borrowObject();
+        pool.returnObject(obj);
+        obj = pool.borrowObject();
+        pool.returnObject(obj);
+        obj = pool.borrowObject();
+        pool.returnObject(obj);
+        // borrow, return, borrow, return 
+        //  FIFO 3,4,5,6,7,0,1,2
+        //  LIFO 7,6,5,4,3,2,1,0
+        // In either case, pointer should be at 6
+        pool.evict();
+        // Should hit 6,7,0 - 0 for second time
+        for (int i = 0; i < 8; i++) {
+            VisitTracker tracker = (VisitTracker) pool.borrowObject();    
+            if (tracker.getId() != 0) {
+                assertEquals("Instance " +  tracker.getId() + 
+                        " visited wrong number of times.",
+                        1, tracker.getValidateCount());
+            } else {
+                assertEquals("Instance " +  tracker.getId() + 
+                        " visited wrong number of times.",
+                        2, tracker.getValidateCount());
+            }
+        } 
+        // Randomly generate a pools with random numTests
+        // and make sure evictor cycles through elements appropriately
+        int[] smallPrimes = {2, 3, 5, 7};
+        Random random = new Random();
+        random.setSeed(System.currentTimeMillis());
+        pool.setMaxIdle(-1);
+        for (int i = 0; i < 4; i++) {
+            pool.setNumTestsPerEvictionRun(smallPrimes[i]);
+            for (int j = 0; j < 5; j++) {
+                pool.clear();
+                int instanceCount = 10 + random.nextInt(20);
+                for (int k = 0; k < instanceCount; k++) {
+                    pool.addObject();
+                }
+
+                // Execute a random number of evictor runs
+                int runs = 10 + random.nextInt(50);
+                for (int k = 0; k < runs; k++) {
+                    pool.evict();
+                }
+
+                // Number of times evictor should have cycled through the pool
+                int cycleCount = (runs * pool.getNumTestsPerEvictionRun())
+                / instanceCount;
+
+                // Look at elements and make sure they are visited cycleCount
+                // or cycleCount + 1 times
+                VisitTracker tracker = null;
+                int visitCount = 0;
+                for (int k = 0; k < instanceCount; k++) {
+                    tracker = (VisitTracker) pool.borrowObject(); 
+                    visitCount = tracker.getValidateCount();                  
+                    assertTrue(visitCount >= cycleCount && 
+                            visitCount <= cycleCount + 1);
+                }
+            }
+        }
+    }
 
     public void testExceptionOnPassivateDuringReturn() throws Exception {
         SimpleFactory factory = new SimpleFactory();        
@@ -165,26 +351,6 @@ public class TestGenericObjectPool extends TestObjectPool {
         pool.close();
     }
 
-    public void testWithInitiallyInvalid() throws Exception {
-        GenericObjectPool pool = new GenericObjectPool(new SimpleFactory(false));
-        pool.setTestOnBorrow(true);
-        try {
-            pool.borrowObject();
-            fail("Expected NoSuchElementException");
-        } catch(NoSuchElementException e) {
-            // expected 
-        }
-    }
-
-    public void testWithSometimesInvalid() throws Exception {
-        GenericObjectPool pool = new GenericObjectPool(new SimpleFactory(true,false));
-        pool.setMaxIdle(10);
-        pool.setTestOnBorrow(true);
-        pool.setTestOnReturn(true);
-        pool.returnObject(pool.borrowObject());  
-        assertEquals(0,pool.getNumIdle());      
-    }
-
     public void testSetFactoryWithActiveObjects() throws Exception {
         GenericObjectPool pool = new GenericObjectPool();
         pool.setMaxIdle(10);
@@ -359,6 +525,10 @@ public class TestGenericObjectPool extends TestObjectPool {
             pool.setTimeBetweenEvictionRunsMillis(11235L);
             assertEquals(11235L,pool.getTimeBetweenEvictionRunsMillis());
         }
+        {
+            pool.setSoftMinEvictableIdleTimeMillis(12135L);
+            assertEquals(12135L,pool.getSoftMinEvictableIdleTimeMillis());
+        }
         {
             pool.setWhenExhaustedAction(GenericObjectPool.WHEN_EXHAUSTED_BLOCK);
             assertEquals(GenericObjectPool.WHEN_EXHAUSTED_BLOCK,pool.getWhenExhaustedAction());
@@ -615,7 +785,13 @@ public class TestGenericObjectPool extends TestObjectPool {
         assertEquals("Should be zero idle, found " + pool.getNumIdle(),0,pool.getNumIdle());
     }
 
-    public void testEvictionSoftMinIdle() throws Exception {
+    /**
+     * This test has a number of problems.
+     * 1. without the wait code in the for loop the create time for each instance
+     * was usually the same due to clock presision.
+     * 2. It's very hard to follow.
+     */
+    public void DISABLEDtestEvictionSoftMinIdle() throws Exception {
         GenericObjectPool pool = null;
         
         class TimeTest extends BasePoolableObjectFactory {
@@ -637,29 +813,37 @@ public class TestGenericObjectPool extends TestObjectPool {
         pool.setMaxActive(5);
         pool.setNumTestsPerEvictionRun(5);
         pool.setMinEvictableIdleTimeMillis(3000L);
+        pool.setTimeBetweenEvictionRunsMillis(250L);
+        pool.setTestWhileIdle(true);
         pool.setSoftMinEvictableIdleTimeMillis(1000L);
         pool.setMinIdle(2);
-
+        
         Object[] active = new Object[5];
         Long[] creationTime = new Long[5] ;
+        long lastCreationTime = System.currentTimeMillis();
         for(int i=0;i<5;i++) {
+            // make sure each instance has a different currentTimeMillis()
+            while (lastCreationTime == System.currentTimeMillis()) {
+                Thread.sleep(1);
+            }
             active[i] = pool.borrowObject();
             creationTime[i] = new Long(((TimeTest)active[i]).getCreateTime());
+            lastCreationTime = creationTime[i].longValue();
         }
         
         for(int i=0;i<5;i++) {
             pool.returnObject(active[i]);
         }
-
-        // Soft evict all but minIdle(2)
-        Thread.sleep(1500L);
-        pool.evict();
-        assertEquals("Idle count different than expected.", 2, pool.getNumIdle());
-
-        // Hard evict the rest.
-        Thread.sleep(2000L);
-        pool.evict();
-        assertEquals("Idle count different than expected.", 0, pool.getNumIdle());
+        
+        try { Thread.sleep(1500L); } catch(Exception e) { }
+        assertTrue("Should be 2 OLD idle, found " + pool.getNumIdle(),pool.getNumIdle() == 2 &&
+                ((TimeTest)pool.borrowObject()).getCreateTime() == creationTime[3].longValue() &&
+                ((TimeTest)pool.borrowObject()).getCreateTime() == creationTime[4].longValue());
+        
+        try { Thread.sleep(2000L); } catch(Exception e) { }
+        assertTrue("Should be 2 NEW idle , found " + pool.getNumIdle(),pool.getNumIdle() == 2 &&
+                ((TimeTest)pool.borrowObject()).getCreateTime() != creationTime[0].longValue() &&
+                ((TimeTest)pool.borrowObject()).getCreateTime() != creationTime[1].longValue());
     }
 
     public void testMinIdle() throws Exception {
@@ -741,17 +925,19 @@ public class TestGenericObjectPool extends TestObjectPool {
         assertTrue("Should be 10 idle, found " + pool.getNumIdle(),pool.getNumIdle() == 10);
     }
 
-    public void testThreaded1() throws Exception {
-        pool.setMaxActive(15);
-        pool.setMaxIdle(15);
-        pool.setMaxWait(1000L);
-        TestThread[] threads = new TestThread[20];
-        for(int i=0;i<20;i++) {
-            threads[i] = new TestThread(pool,100,50);
+    /**
+     * Kicks off <numThreads> test threads, each of which will go through
+     * <iterations> borrow-return cycles with random delay times <= delay
+     * in between.
+     */
+    public void runTestThreads(int numThreads, int iterations, int delay) {
+        TestThread[] threads = new TestThread[numThreads];
+        for(int i=0;i<numThreads;i++) {
+            threads[i] = new TestThread(pool,iterations,delay);
             Thread t = new Thread(threads[i]);
             t.start();
         }
-        for(int i=0;i<20;i++) {
+        for(int i=0;i<numThreads;i++) {
             while(!(threads[i]).complete()) {
                 try {
                     Thread.sleep(500L);
@@ -764,6 +950,33 @@ public class TestGenericObjectPool extends TestObjectPool {
             }
         }
     }
+    
+    public void testThreaded1() throws Exception {
+        pool.setMaxActive(15);
+        pool.setMaxIdle(15);
+        pool.setMaxWait(1000L);
+        runTestThreads(20, 100, 50);
+    }
+    
+    /**
+     * Verifies that maxActive is not exceeded when factory destroyObject
+     * has high latency, testOnReturn is set and there is high incidence of
+     * validation failures. 
+     */
+    public void testMaxActiveInvariant() throws Exception {
+        int maxActive = 15;
+        SimpleFactory factory = new SimpleFactory();
+        factory.setEvenValid(false);     // Every other validation fails
+        factory.setDestroyLatency(100);  // Destroy takes 100 ms
+        factory.setMaxActive(maxActive); // (makes - destroys) bound
+        factory.setValidationEnabled(true);
+        pool = new GenericObjectPool(factory);
+        pool.setMaxActive(maxActive);
+        pool.setMaxIdle(-1);
+        pool.setTestOnReturn(true);
+        pool.setMaxWait(1000L);
+        runTestThreads(5, 10, 50);
+    }
 
     class TestThread implements Runnable {
         java.util.Random _random = new java.util.Random();
@@ -772,6 +985,7 @@ public class TestGenericObjectPool extends TestObjectPool {
         boolean _failed = false;
         int _iter = 100;
         int _delay = 50;
+        boolean _randomDelay = true;
 
         public TestThread(ObjectPool pool) {
             _pool = pool;
@@ -787,6 +1001,14 @@ public class TestGenericObjectPool extends TestObjectPool {
             _iter = iter;
             _delay = delay;
         }
+        
+        public TestThread(ObjectPool pool, int iter, int delay,
+                boolean randomDelay) {
+            _pool = pool;
+            _iter = iter;
+            _delay = delay;
+            _randomDelay = randomDelay;
+        }
 
         public boolean complete() {
             return _complete;
@@ -798,8 +1020,10 @@ public class TestGenericObjectPool extends TestObjectPool {
 
         public void run() {
             for(int i=0;i<_iter;i++) {
+                long delay = 
+                    _randomDelay ? (long)_random.nextInt(_delay) : _delay;
                 try {
-                    Thread.sleep((long)_random.nextInt(_delay));
+                    Thread.sleep(delay);
                 } catch(Exception e) {
                     // ignored
                 }
@@ -807,13 +1031,14 @@ public class TestGenericObjectPool extends TestObjectPool {
                 try {
                     obj = _pool.borrowObject();
                 } catch(Exception e) {
+                    e.printStackTrace();
                     _failed = true;
                     _complete = true;
                     break;
                 }
 
                 try {
-                    Thread.sleep((long)_random.nextInt(_delay));
+                    Thread.sleep(delay);
                 } catch(Exception e) {
                     // ignored
                 }
@@ -830,6 +1055,7 @@ public class TestGenericObjectPool extends TestObjectPool {
     }
 
     public void testFIFO() throws Exception {
+        pool.setLifo(false);
         pool.addObject(); // "0"
         pool.addObject(); // "1"
         pool.addObject(); // "2"
@@ -841,6 +1067,20 @@ public class TestGenericObjectPool extends TestObjectPool {
         assertEquals("returned", "r", pool.borrowObject());
         assertEquals("new-4", "4", pool.borrowObject());
     }
+    
+    public void testLIFO() throws Exception {
+        pool.setLifo(true);
+        pool.addObject(); // "0"
+        pool.addObject(); // "1"
+        pool.addObject(); // "2"
+        assertEquals("Youngest", "2", pool.borrowObject());
+        assertEquals("Middle", "1", pool.borrowObject());
+        assertEquals("Oldest", "0", pool.borrowObject());
+        assertEquals("new-3", "3", pool.borrowObject());
+        pool.returnObject("r");
+        assertEquals("returned", "r", pool.borrowObject());
+        assertEquals("new-4", "4", pool.borrowObject());
+    }
 
     public void testAddObject() throws Exception {
         assertEquals("should be zero idle", 0, pool.getNumIdle());
@@ -853,78 +1093,18 @@ public class TestGenericObjectPool extends TestObjectPool {
 		pool.returnObject(obj);
 		assertEquals("should be one idle", 1, pool.getNumIdle());
 		assertEquals("should be zero active", 0, pool.getNumActive());
+
+        ObjectPool op = new GenericObjectPool();
+        try {
+            op.addObject();
+            fail("Expected IllegalStateException when there is no factory.");
+        } catch (IllegalStateException ise) {
+            //expected
+        }
+        op.close();
     }
     
-    private List testFactorySequenceStates = new ArrayList(5);
-    public void testFactorySequence() throws Exception {
-        // setup
-        // We need a factory that tracks method call sequence.
-        PoolableObjectFactory pof = new PoolableObjectFactory() {
-            public Object makeObject() throws Exception {
-                testFactorySequenceStates.add("makeObject");
-                return new Object();
-            }
-
-            public void activateObject(Object obj) throws Exception {
-                testFactorySequenceStates.add("activateObject");
-            }
-
-            public boolean validateObject(Object obj) {
-                testFactorySequenceStates.add("validateObject");
-                return true;
-            }
-
-            public void passivateObject(Object obj) throws Exception {
-                testFactorySequenceStates.add("passivateObject");
-            }
-
-            public void destroyObject(Object obj) throws Exception {
-                testFactorySequenceStates.add("destroyObject");
-            }
-        };
-
-        GenericObjectPool pool = new GenericObjectPool(pof);
-        pool.setTestOnBorrow(true);
-        pool.setTestOnReturn(true);
-
-        // check the order in which the factory is called during borrow
-        testFactorySequenceStates.clear();
-        Object o = pool.borrowObject();
-        List desiredSequence = Arrays.asList(new String[] {
-                "makeObject",
-                "activateObject",
-                "validateObject"
-        });
-        assertEquals("Wrong sequence", desiredSequence, testFactorySequenceStates);
-
-        // check the order in which the factory is called when returning an object
-        testFactorySequenceStates.clear();
-        pool.returnObject(o);
-        desiredSequence = Arrays.asList(new String[] {
-                "validateObject",
-                "passivateObject"
-        });
-        assertEquals("Wrong sequence", desiredSequence, testFactorySequenceStates);
-
-        // check the order in which the factory is called during borrow again
-        testFactorySequenceStates.clear();
-        o = pool.borrowObject();
-        desiredSequence = Arrays.asList(new String[] {
-                "activateObject",
-                "validateObject"
-        });
-        assertEquals("Wrong sequence", desiredSequence, testFactorySequenceStates);
-
-        // check the order in which the factory is called when invalidating an object
-        testFactorySequenceStates.clear();
-        pool.invalidateObject(o);
-        desiredSequence = Arrays.asList(new String[] {
-                "destroyObject"
-        });
-        assertEquals("Wrong sequence", desiredSequence, testFactorySequenceStates);
-    }
-
-    private GenericObjectPool pool = null;
+    protected GenericObjectPool pool = null;
 
     private void assertConfiguration(GenericObjectPool.Config expected, GenericObjectPool actual) throws Exception {
         assertEquals("testOnBorrow",expected.testOnBorrow,actual.getTestOnBorrow());
@@ -963,9 +1143,36 @@ public class TestGenericObjectPool extends TestObjectPool {
         public void setThrowExceptionOnPassivate(boolean bool) {
             exceptionOnPassivate = bool;
         }
-    
-        public Object makeObject() { return String.valueOf(makeCounter++); }
-        public void destroyObject(Object obj) { }
+        public void setMaxActive(int maxActive) {
+            this.maxActive = maxActive;
+        }
+        public void setDestroyLatency(long destroyLatency) {
+            this.destroyLatency = destroyLatency;
+        }
+        public void setMakeLatency(long makeLatency) {
+            this.makeLatency = makeLatency;
+        }
+        public Object makeObject() { 
+            synchronized(this) {
+                activeCount++;
+                if (activeCount > maxActive) {
+                    throw new IllegalStateException(
+                        "Too many active instances: " + activeCount);
+                }
+            }
+            if (makeLatency > 0) {
+                doWait(makeLatency);
+            }
+            return String.valueOf(makeCounter++);
+        }
+        public void destroyObject(Object obj) {
+            if (destroyLatency > 0) {
+                doWait(destroyLatency);
+            }
+            synchronized(this) {
+                activeCount--;
+            }
+        }
         public boolean validateObject(Object obj) {
             if (enableValidation) { 
                 return validateCounter++%2 == 0 ? evenValid : oddValid; 
@@ -988,11 +1195,15 @@ public class TestGenericObjectPool extends TestObjectPool {
         }
         int makeCounter = 0;
         int validateCounter = 0;
+        int activeCount = 0;
         boolean evenValid = true;
         boolean oddValid = true;
         boolean exceptionOnPassivate = false;
         boolean exceptionOnActivate = false;
         boolean enableValidation = true;
+        long destroyLatency = 0;
+        long makeLatency = 0;
+        int maxActive = Integer.MAX_VALUE;
 
         public boolean isThrowExceptionOnActivate() {
             return exceptionOnActivate;
@@ -1009,15 +1220,21 @@ public class TestGenericObjectPool extends TestObjectPool {
         public void setValidationEnabled(boolean b) {
             enableValidation = b;
         }
+        
+        private void doWait(long latency) {
+            try {
+                Thread.sleep(latency);
+            } catch (InterruptedException ex) {
+                // ignore
+            }
+        }
     }
-
     protected boolean isLifo() {
- 
-        return false;
+        return true;
     }
 
     protected boolean isFifo() {
-        return true;
+        return false;
     }
 }
 
